package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-24 14:02
 * @Modify:
 **/
@RestController
@Api(description = "系统用户管理服务")
@RequestMapping(PlatformConstWords.BASE_URL +"/personnel")
public class PersonnelController  extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(PersonnelController.class);

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysFileService fileService;


    @PostMapping(value="/user-detail/insert")
    @ApiOperation(value="添加用户信息",notes="返回内容添加用户信息")
    public ResultBean insertUserDetailModel(UserDetailModel model) {
        try {
            SysUser user = model.getSysUser();
            SysUserInfo info = model.getSysUserInfo();

            checkUserDetailModel(model, false);
            personnelService.checkSysUserOnly(user);

            SysDepartment infoDepartment = personnelService.getSysDepartmentById(Long.parseLong(model.getInfoDepartmentId()));
            if(infoDepartment == null){
                ValidatorBean.checkNotNull(infoDepartment,"用户主部门不存在");
            }
            SysOrganize infoOrganize = personnelService.getSysOrganizeById(infoDepartment.getOrganizeId());
            SysPosition position = personnelService.getSysPositionById(Long.parseLong(model.getInfoPositionId()));
            if(infoDepartment == null){
                ValidatorBean.checkNotNull(position,"用户主岗位存在");
            }

            SysDepartment userDepartment = personnelService.getSysDepartmentById(Long.parseLong(model.getUserDepartmentId()));
            if(userDepartment == null){
                ValidatorBean.checkNotNull(userDepartment,"账号主部门不存在");
            }
            SysOrganize userOrganize = personnelService.getSysOrganizeById(userDepartment.getOrganizeId());

            info.setDepartmentId(infoDepartment.getId());
            info.setDepartmentNameRdd(infoDepartment.getName());
            info.setOrganizeId(infoOrganize.getId());
            info.setOrganizeNameRdd(infoOrganize.getName());
            info.setOrganizeCode(infoOrganize.getOrganizeCode());
            info.setPositionId(position.getId());
            info.setPositionNameRdd(position.getName());
            personnelService.saveSysUserInfo(info);

            user.setDepartmentId(userDepartment.getId());
            user.setDepartmentNameRdd(userDepartment.getName());
            user.setOrganizeId(userOrganize.getId());
            user.setOrganizeNameRdd(userOrganize.getName());
            user.setOrganizeCode(userOrganize.getOrganizeCode());
            user.setUserInfoId(info.getId());
            user.setUserLoginPassword(EncryptTool.hexMD5(user.getUserLoginPassword()));
            personnelService.saveSysUser(user);

            info.setUserId(user.getId());
            personnelService.saveSysUserInfo(info);

            // 关系 刷新
            refreshRef(user, info, model);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @PutMapping(value="/user-detail/update")
    @ApiOperation(value="添加用户信息",notes="返回内容添加用户信息")
    public ResultBean updateUserDetailModel(UserDetailModel model) {
        try {
            SysUser user = model.getSysUser();
            SysUserInfo info = model.getSysUserInfo();

            checkUserDetailModel(model, true);
            personnelService.checkSysUserOnly(user);

            initUserDetail(info,user);

            SysDepartment infoDepartment = personnelService.getSysDepartmentById(Long.parseLong(model.getInfoDepartmentId()));
            if(infoDepartment == null){
                ValidatorBean.checkNotNull(infoDepartment,"用户主部门不存在");
            }
            SysOrganize infoOrganize = personnelService.getSysOrganizeById(infoDepartment.getOrganizeId());
            SysPosition position = personnelService.getSysPositionById(Long.parseLong(model.getInfoPositionId()));
            if(infoDepartment == null){
                ValidatorBean.checkNotNull(position,"用户主岗位存在");
            }

            SysDepartment userDepartment = personnelService.getSysDepartmentById(Long.parseLong(model.getUserDepartmentId()));
            if(userDepartment == null){
                ValidatorBean.checkNotNull(userDepartment,"账号主部门不存在");
            }
            SysOrganize userOrganize = personnelService.getSysOrganizeById(userDepartment.getOrganizeId());

            info.setDepartmentId(infoDepartment.getId());
            info.setDepartmentNameRdd(infoDepartment.getName());
            info.setOrganizeId(infoOrganize.getId());
            info.setOrganizeNameRdd(infoOrganize.getName());

            user.setDepartmentId(userDepartment.getId());
            user.setDepartmentNameRdd(userDepartment.getName());
            user.setOrganizeId(userOrganize.getId());
            user.setOrganizeNameRdd(userOrganize.getName());

            // 关系 刷新
            refreshRef(user, info, model);

            personnelService.saveSysUser(user);
            personnelService.saveSysUserInfo(info);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @GetMapping(value = "/user-info/get-detail/{id}")
    @ApiOperation(value = "获取用户信息", notes = "查询根据ID查询用户相关所有信息")
    public ResultBean getUserInfoById(@PathVariable("id") String id){
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "角色id 不能为空");
            Map<String,Object> resultMap = new HashMap<>();


            SysUserInfo info = personnelService.getSysUserInfoById(Long.parseLong(id));
            SysUser user = personnelService.getSysUserById(info.getUserId());

            List<SysRefUserInfoDepartment> refUserInfoDepartmentList = personnelService.findSysRefUserInfoDepartmentById(info.getId());
            List<SysRefUserPosition> refUserPositionList = personnelService.findSysRefUserPositionById(info.getId());
            List<SysRefUserDepartment> refUserDepartmentList = personnelService.findSysRefUserDepartmentById(user.getId());
            List<SysRefUserRole> refUserRoleList = personnelService.findSysRefUserRoleById(user.getId());
            SysFile image = info.getUserImageId() != null ? fileService.getSysFileById(info.getUserImageId()) : null;

            resultMap.put("user",user);
            resultMap.put("refUserInfoDepartmentList",refUserInfoDepartmentList);
            resultMap.put("refUserPositionList",refUserPositionList);
            resultMap.put("refUserDepartmentList",refUserDepartmentList);
            resultMap.put("refUserRoleList",refUserRoleList);
            resultMap.put("image",image);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultMap(resultMap);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value="/user-info/delete/{id}")
    @ApiOperation(value="删除用户信息",notes="根据用户信息id删除信息")
    public ResultBean deleteSysUserInfo(@PathVariable("id") String id){
        try {
            //条件验证
            ValidatorBean.checkNotNull(id,"id为空");

            SysUserInfo info = personnelService.getSysUserInfoById(Long.parseLong(id));
            if(info != null){
                personnelService.deleteSysUserInfo(new Long[]{info.getId()},getSessionUser().getUserName());
                personnelService.deleteSysUser(new Long[]{info.getUserId()},getSessionUser().getUserName());

                return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else{
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @DeleteMapping(value = "/user-info/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysUserInfoByIds(String[] ids){
        try{
            ids = ConvertBean.modelSafeArrayNumber(ids,true);
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户信息。")
                        .build();
            }

            List<SysUserInfo> list = personnelService.findSysUserInfoByIds(StringTool.getArrayLong(ids));

            if(list != null && list.size() > 0){
                Long[] infoIds = new Long[list.size()];
                Long[] userIds = new Long[list.size()];

                for (int i = 0; i < list.size(); i++) {
                    infoIds[i] = list.get(i).getId();
                    userIds[i] = list.get(i).getUserId();
                }

                personnelService.deleteSysUserInfo(infoIds,getSessionUser().getUserName());
                personnelService.deleteSysUser(userIds,getSessionUser().getUserName());

                return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else{
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     * 数据必填校验
     * @param model
     * @param isCheckId
     * @throws Exception
     */
    private void checkUserDetailModel(UserDetailModel model,boolean isCheckId)throws Exception{
        if(isCheckId){
            ValidatorBean.beginValid(model)
                    .notNull("userInfoId", model.getUserInfoId())
                    .notNull("userId", model.getUserId());
        }else {
            ValidatorBean.beginValid(model).notNull("userLoginPassword", model.getUserLoginPassword());
        }

        ValidatorBean.beginValid(model)
                .notNull("infoName", model.getInfoName())
                .notNull("infoEmpNo", model.getInfoEmpNo())
                .notNull("infoPositionId", model.getInfoPositionId())
                .notNull("infoDepartmentId", model.getInfoDepartmentId())
                .notNull("userLoginName", model.getUserLoginName())
                .notNull("userEmail", model.getUserEmail())
                .notNull("userPhone", model.getUserPhone());
    }

    /**
     * 用户信息初始化
     * @param info
     * @param user
     * @throws Exception
     */
    private void initUserDetail(SysUserInfo info ,SysUser user)throws Exception{
        if(StringUtils.isBlank(info.getUserJoinDate())){
            info.setUserJoinDate(TimeTool.getToday());
        }

        info.setUserInfoStatus(CommonEnumUtil.USER_INFO_STATUS.INCUMBENT.getValue());
        info.setUserLoginNum(0);
        ConvertBean.modelInitialize(info, AuthUtil.getSessionUser());

        user.setUserType(CommonEnumUtil.USER_TYPE.USER.getValue());
        user.setUserStatus(CommonEnumUtil.USER_STATUS.ENABLE.getValue());
        user.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
        user.setUserLoginNum(0);
        ConvertBean.modelInitialize(user, AuthUtil.getSessionUser());
    }

    /**
     * 关系刷新
     * @param user
     * @param info
     * @param model
     */
    private void refreshRef(SysUser user,SysUserInfo info,UserDetailModel model){
        // 刷新账号 部门关系
        personnelService.refreshRefSysUserDepartment(user.getId(), StringTool.getArrayLong(model.getUserDepartmentIdList()));
        // 刷新账号 角色关系
        personnelService.refreshRefSysUserRole(user.getId(),StringTool.getArrayLong(model.getUserRoleIdList()));

        // 刷新用户 部门关系
        personnelService.refreshRefSysUserInfoDepartment(info.getId(),StringTool.getArrayLong(model.getInfoDepartmentIdList()));
        // 刷新用户 岗位关系
        personnelService.refreshRefSysUserInfoPosition(info.getId(),StringTool.getArrayLong(model.getInfoPositionIdList()));
    }
}
